package BsUtils
import(
	_ "github.com/go-sql-driver/mysql"
	"database/sql"
    "strings"
	"fmt"
	"time"
)

func setdbinfo() (*sql.DB) {
    dbusername, dbpassword, dbhostsip, dbname,_,_:=ConfigYamlParse()
	dbinfo := strings.Join([]string{dbusername, ":", dbpassword, "@tcp(", dbhostsip, ")/", dbname, "?charset=utf8"}, "")
	db,_ := sql.Open("mysql",dbinfo)
	// 设置最大连接数
	db.SetConnMaxLifetime(100)
	// 设置数据库最大的闲置连接数
	db.SetMaxIdleConns(10)
	return db
}

// 查询扫描记录
func SelectScanNums()(int,int,int,int){
    db:=setdbinfo()
    type Info struct {
        system_nums int `db:"system_nums"`
        baseline_nums int `db:"baseline_nums"`
        kernel_nums int `db:"kernel_nums"`
        fastscan_nums int `db:"fastscan_nums"`
    }
    var info Info
    selectSqlStr := "SELECT SUM(system_nums), SUM(baseline_nums), SUM(kernel_nums), SUM(fastscan_nums) FROM info"
    err := db.QueryRow(selectSqlStr).Scan(&info.system_nums,&info.baseline_nums,&info.kernel_nums,&info.fastscan_nums)
	checkerr(err)
    return info.system_nums,info.baseline_nums,info.kernel_nums,info.fastscan_nums
}

// add user
func AddUserDB(username string, password string)bool {
    result:=selectUsernemWhether(username)
    if result==false{
        return false
    }
	db:=setdbinfo()
    password=PasswordEncryption(password)
    var status bool = true
	sqlStr := "insert into User(username,password) values (?,?)"
	exec, err := db.Exec(sqlStr, username,password)
    if err != nil {
		fmt.Println(err)
        status=false
	}
	//插入数据的id
	_, err = exec.LastInsertId()
	if err != nil {
		fmt.Println("id err:",err)
        status=false
	}

    return status
}

// select username whether or not
func selectUsernemWhether(username string)bool {
	db:=setdbinfo()
    type UserId struct {
        id int `db:"id"`
    }
    var userid UserId
    selectSqlStr := "SELECT id FROM User where username = ?"
    err := db.QueryRow(selectSqlStr, username).Scan(&userid.id)
	checkerr(err)
    if userid.id!=0 {
        return false
    }else{
        return true
    }
}
// 查询该id用户各个扫描数据情况
func scanNums(id int)(int, int, int, int) {

	db:=setdbinfo()
    //>>>dbInfo
    type Info struct {
        system_nums int `db:"system_nums"`
        baseline_nums int `db:"baseline_nums"`
        kernel_nums int `db:"kernel_nums"`
        fastscan_nums int `db:"fastscan_nums"`
    }
    var info Info
    // 预编译处理查询数据防止sql注入 -- 查询该id用户各个扫描数据情况☞
	selectIdSqlStr := "SELECT system_nums,baseline_nums,kernel_nums,fastscan_nums FROM info where id = ?" 
    stmt, err := db.Prepare(selectIdSqlStr)
	checkerr(err)
    defer stmt.Close()

    rows, err := stmt.Query(id)
    for rows.Next(){
        err = rows.Scan(&info.system_nums, &info.baseline_nums, &info.kernel_nums, &info.fastscan_nums)
        checkerr(err)
    }
    return info.system_nums, info.kernel_nums, info.baseline_nums, info.fastscan_nums
}

// scan nums count
func ScanNumsAdd(username string, scanType string, host string){
	db:=setdbinfo()

    now := time.Now()
	// 格式化日期时间
	ScanTime := now.Format("2006/01/02 15:04:05")

    // 获取用户id
    id:=getId(username)

    // // 获取扫描数据
    system_nums, kernel_nums, baseline_nums, fastscan_nums:=scanNums(id)
    var constBool bool = false
    if system_nums == 0 && kernel_nums == 0 && baseline_nums == 0 && fastscan_nums == 0{
        constBool = true
    }
    if scanType == "baseline" {
        baseline_nums=baseline_nums+1
    }else if scanType == "kernel" {
        kernel_nums=kernel_nums+1
    }else if scanType == "system" {
        system_nums=system_nums+1
    }else if scanType == "fastscan" {
        fastscan_nums=fastscan_nums+1
    }
    if constBool {
        insertNums := "insert into info(baseline_nums,kernel_nums,system_nums,fastscan_nums,host,time,id) values (?,?,?,?,?,?,?)"
        _, err := db.Exec(insertNums, baseline_nums, kernel_nums, system_nums, fastscan_nums, host, ScanTime, id)
        checkerr(err)
    }else{
        updateNums := "update info set baseline_nums = ?, kernel_nums = ?, system_nums = ?, fastscan_nums = ?, host = ?, time = ? where id = ? "
        exec, err := db.Exec(updateNums, baseline_nums, kernel_nums, system_nums, fastscan_nums, host, ScanTime, id)
        checkerr(err)
        
        _, err = exec.RowsAffected()
        checkerr(err)
        // fmt.Println("修改成功的行数",affected)
    }
}

// Get User ID
func getId(username string)int{
	db:=setdbinfo()
    //>>>dbInfo
	type User struct {
		id int `db:"id"`
	}
    // fmt.Println(username)
    var user User
    // 预编译处理查询数据防止sql注入 -- 查询ID☞
	selectIdSqlStr := "SELECT id FROM User where username = ?" 
    stmt, err := db.Prepare(selectIdSqlStr)
	checkerr(err)
    defer stmt.Close()
    rows, err := stmt.Query(username)
    for rows.Next(){
        err = rows.Scan(&user.id)
        checkerr(err)
    }
    return user.id
}


// mysql - username and password check
func LoginQuery(username string , password string) bool{
	db:=setdbinfo()

    //>>>dbInfo
	type info struct {
		id int `db:"id"`
        dbusername string `db: "username"`
        dbpassword string `db: "password"`
	}
    var s info
    //预编译处理查询数据防止sql注入
	sqlStr := "SELECT password FROM User where username = ?" 
    stmt, err := db.Prepare(sqlStr)
	checkerr(err)
    defer stmt.Close()
    rows, err := stmt.Query(username)
    for rows.Next(){
        err = rows.Scan(&s.dbpassword)
        checkerr(err)
    }
    if password != s.dbpassword{
        return false
    }else{
        return true
    }
}

// 添加登录时间
func AddLoginTimeAndIp(username string,ip string) {
    uid:=getId(username)
    todaystr := time.Now().Format("2006-01-02 15:04:05")
    // end_time:= time.Now().Format("15:04:05")

	db:=setdbinfo()

	sqlStr := "UPDATE User SET login_time = ? ,ip = ? where id = ?"
	_, err := db.Exec(sqlStr, todaystr,ip, uid)
    if err != nil {
		fmt.Println(err)
        return
	}
}


// err check
func checkerr(err error){  
	if err!=nil{
		fmt.Printf("%c[%d;%d;%dm%s%c[0m", 0x1B, 0, 0, 31, "", 0x1B)
		fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 31, err, 0x1B)
        return
	}
 
}