package RemoteCheck
import (
	"fmt"
	"github.com/pkg/sftp"
	"golang.org/x/crypto/ssh"
	"io/ioutil"
	"encoding/json"
	// "regexp"
	"strconv"
	// "bufio"
	// "io"
	"strings"
	"path/filepath"
	"log"
	"net"
	"path"
	"os"
	"time"
)
// Connected configuration
type ClientConfig struct {
	Host       string       //ip
	Port       int        // 端口
	Username   string       //用户名
	Password   string       //密码
	sshClient  *ssh.Client  //ssh client
	sftpClient *sftp.Client //sftp client
	LastResult string       //最近一次运行的结果
}

type Json struct { 
	Time 				string  `json:"RCExploredTimeAt"`
	ExecPocNums 		int 	`json:"RCExecPocNums"`
	RCRepairedNums 		int 	`json:"RCRepairedNums"`
	RCNotFixedNums 		int		`json:"RCNotFixedNums"`
	RCNotExecPocNums 	int		`json:"RCNotExecPocNums"`
	VbNums				int		`json:"VbNums"`
	VbLevel_hight		int		`VbLevel_hight"`
	VbLevel_mid			int		`json:"VbLevel_mid"`
	VbLevel_low			int		`json:"VbLevel_low"`
	VbLevel_ignore		int		`json:"VbLevel_ignore"`

}
var remoteUpdateStatus int
var remoteStatus int

// Create ssh/ftp client
func (cliConf *ClientConfig) createClient(host string, port int, username, password string)error {
	var (
		sshClient  *ssh.Client
		sftpClient *sftp.Client
		err        error
	)
	cliConf.Host = host
	cliConf.Port = port
	cliConf.Username = username
	cliConf.Password = password
	cliConf.Port = port
	config := ssh.ClientConfig{
		User: cliConf.Username,
		Auth: []ssh.AuthMethod{ssh.Password(password)},
		HostKeyCallback: func(hostname string, remote net.Addr, key ssh.PublicKey) error {
			return nil
		},
		Timeout: 10 * time.Second,
	}
	addr := fmt.Sprintf("%s:%d", cliConf.Host, cliConf.Port)
	if sshClient, err = ssh.Dial("tcp", addr, &config); err != nil {
		log.Println("error occurred:", err)
		return err
	}
	cliConf.sshClient = sshClient
	//The sshClient is obtained. Next, use the sshClient to build the sftpClient
	if sftpClient, err = sftp.NewClient(sshClient); err != nil {
		log.Println("error occurred:", err)
		return err
	}
	cliConf.sftpClient = sftpClient
	return err
}

//ssh command exec
func (cliConf *ClientConfig) RunShell(shell string) string {
	var (
		session *ssh.Session
		err     error
	)
	//Get a session that is used to remotely perform operations
	if session, err = cliConf.sshClient.NewSession(); err != nil {
		log.Println("error occurred:", err)
	}
	//Exec shell
	// shellExec:=strings.Replace(shell,"!","\\!",-1)
	if output, err := session.CombinedOutput(shell); err != nil {
		fmt.Println(shell,err)
		// log.Fatalln("error occurred:", err)
		cliConf.LastResult = "0"
	} else {
		cliConf.LastResult = string(output)
	}
	return cliConf.LastResult
}

// Transfer dir
func (cliConf *ClientConfig) uploadDirectory(localPath string, remotePath string){

	localFiles, err := ioutil.ReadDir(localPath)
	if err != nil {
		fmt.Println("read dir list fail ", err)
		return
	}
	for _, backupDir := range localFiles {
		localFilePath := path.Join(localPath, backupDir.Name())
		remoteFilePath := path.Join(remotePath, backupDir.Name())
		if backupDir.IsDir() {
			cliConf.sftpClient.Mkdir(remoteFilePath)
	    	cliConf.uploadDirectory(localFilePath, remoteFilePath)
	 	} else {
			cliConf.uploadFile(path.Join(localPath, backupDir.Name()), remotePath)
	 	}
	}
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 32, remotePath, 0x1B)
}

// Transfer files
func (cliConf *ClientConfig) uploadFile(localFilePath string, remotePath string) {
	srcFile, err := os.Open(localFilePath)
	if err != nil {
	 	fmt.Println("os.Open error : ", localFilePath)
	 	log.Println(err)
		return
	}
	defer srcFile.Close()
	var remoteFileName = path.Base(localFilePath)
	dstFile, err := cliConf.sftpClient.Create(path.Join(remotePath, remoteFileName))
	if err != nil {
	 	fmt.Println("sftpClient.Create error : ", path.Join(remotePath, remoteFileName))
	 	log.Println(err)
		return
	}
	defer dstFile.Close()
	ff, err := ioutil.ReadAll(srcFile)
	if err != nil {
	 	fmt.Println("ReadAll error : ", localFilePath)
	 	log.Println(err)
		return
	}
	dstFile.Write(ff)
   }

// Get remote report
func (cliConf *ClientConfig) Download(srcPath, dstPath string)bool{
	srcFile, err:= cliConf.sftpClient.Open(srcPath) //remote
	if err != nil {
		fmt.Println(err)
		return false
	 }
	dstFile, err:= os.Create(dstPath) //loacl
	if err != nil {
		fmt.Println(err)
		return false
	 }
	defer func() {
		_ = srcFile.Close()
		_ = dstFile.Close()
	}()
	if _, err := srcFile.WriteTo(dstFile); err != nil {
		fmt.Println("error occurred", err)
		return false
	}
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 32,"File download completed\n" , 0x1B)
	return true
}


type currentDirInfo struct {
	currentDirs  []string
	currentFiles []string
 }
 
 func getCurrentDir(path string) (currentDirInfo, error) {
	infos, err := ioutil.ReadDir(path)
	if err != nil {
	   return currentDirInfo{}, nil
	}
	var currentDirs []string
	var currentFiles []string
	for _, info := range infos {
	   fmt.Printf("Name:%-30s    Size:%-10d字节    Modtime:%s    IsDir:%v\n", info.Name(), info.Size(), info.ModTime().Format("2006-01-02 03:04:05"), info.IsDir())
	   fmt.Printf("\n")
	   abs, err := filepath.Abs(info.Name())
	   if err != nil {
		  return currentDirInfo{}, nil
	   }
	   if info.IsDir() {
		  currentDirs = append(currentDirs, abs)
	   } else {
		  currentFiles = append(currentFiles, abs)
	   }
	}
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"=======================当前文件夹==============================\n" , 0x1B)
	for _, v := range currentDirs {
	   fmt.Printf("当前文件夹:%s\n", v)
	   fmt.Printf("\n")
	}
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"========================当前文件==============================\n" , 0x1B)
	for _, v := range currentFiles {
	   fmt.Printf("当前文件:%s\n", v)
	   fmt.Printf("\n")
	}
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"========================开始传输==============================\n" , 0x1B)
	return currentDirInfo{currentDirs: currentDirs, currentFiles: currentFiles}, nil
 }
 
func hostKeyCallback(hostname string, remote net.Addr, key ssh.PublicKey) error {
	return nil
}

//文件检测是否存在
func PathExists(path string)(bool,error){
    _,err := os.Stat(path)
    if err == nil{
        return true,nil
    }
    //isnotexist来判断，是不是不存在的错误
    if os.IsNotExist(err){	//如果返回的错误类型使用os.isNotExist()判断为true，说明文件或者文件夹不存在
        return false,nil
    }
    return false,err//如果有错误了，但是不是不存在的错误，所以把这个错误原封不动的返回
}

func RemoteScan(host string,port int,user string,passwd string,RemoteArg []string,RootPasswd string) (int,string){
	
	cliConf := new(ClientConfig)
	errConnet:=cliConf.createClient(host, port, user, passwd)
	if errConnet !=nil{
		return 1000,""
	}
	framework:=cliConf.RunShell("uname -m")
	framework=strings.TrimSpace(framework)
	if framework == "sw_64"{
		return 1001,""
	}
	if framework == "loongarch64"{
		return 1001,""
	}
	checkGenmaiDir:=cliConf.RunShell("ls genmai")
	if checkGenmaiDir =="0"{
		cliConf.RunShell("mkdir genmai; cd genmai ; mkdir data ;mkdir src")
		cliConf.RunShell("cd genmai/data ; mkdir BaseLine ;mkdir SandboxViPyEnv; mkdir SystemPocs; mkdir dic; mkdir Report;mkdir static; mkdir KernelPocs")
	}
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"========================Data==============================\n" , 0x1B)
	_, err := getCurrentDir("../data/")
	if err != nil {
	   log.Println("Get data dir err")
	}
	if checkGenmaiDir == "0" {
		cliConf.uploadDirectory("../data/BaseLine", "/home/"+user+"/genmai/data/BaseLine")
		cliConf.uploadDirectory("../data/SandboxViPyEnv", "/home/"+user+"/genmai/data/SandboxViPyEnv")
		cliConf.uploadDirectory("../data/SystemPocs", "/home/"+user+"/genmai/data/SystemPocs")
		cliConf.uploadDirectory("../data/static", "/home/"+user+"/genmai/data/static")
		cliConf.uploadDirectory("../data/KernelPocs", "/home/"+user+"/genmai/data/KernelPocs")
		cliConf.uploadDirectory("../data/dic", "/home/"+user+"/genmai/data/dic")
	}
	fmt.Printf("\n")
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"========================SRC==============================\n" , 0x1B)
	_, err  = getCurrentDir(".")
	if err != nil {
		log.Println("Get current dir err")
	 }
	// get system info
	// _,framework := genmai.GetCurrentFw()

	if framework == "mips64"{
		byArch:=cliConf.RunShell("lscpu | awk 'NR==2 {print $2}'")
		byArch=strings.TrimSpace(byArch)
		if byArch == "Little"{
			framework="mips64le"
		}
	}

	binMain:="main-"+framework
	if checkGenmaiDir == "0" {
		cliConf.uploadDirectory(".", "/home/"+user+"/genmai/src")
		// cliConf.uploadDirectory("config.yaml", "/home/"+user+"/genmai/src")
	}
	fmt.Printf("\n")
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"========================环境配置==========================\n" , 0x1B)
	command_kysec:="kysec_set -rn exectl -v verified /home/"+user+"/genmai"
	cliConf.RunShell(command_kysec)
	commandChmod:="chmod +x genmai/src/"+binMain
	cliConf.RunShell(commandChmod)

		// ------------------------- 转译处理 -------------------------
		// regular := regexp.MustCompile(`([!$%^&*()+\[\]{}\\|;':"<>?,./])`)
		// passwd=regular.ReplaceAllString(passwd, `\\$1`)
		// user=regular.ReplaceAllString(user, `\\$1`)
		// RootPasswd=regular.ReplaceAllString(RootPasswd, `\\$1`)
		// ------------------------- 转译处理 -------------------------

		// to do 需要修改将单引号和双引号转译

	// --------------------------- root 配置 -------------------------
	command_checkIfSudo:="echo '"+passwd+"' | sudo -S cat /etc/shadow"
	r:=cliConf.RunShell(command_checkIfSudo)
	rootChangeVul:=0 
	if 	r != "0"{
		rootChangeVul = 1
		command_copy_shadow := "echo "+passwd+" | sudo -S cp /etc/shadow /etc/shadow.bak"
		cliConf.RunShell(command_copy_shadow)
		command_changeRootPasswd:="echo -e '"+passwd+"\n"+passwd+"\n"+passwd+"' | sudo -S passwd root"
		cliConf.RunShell(command_changeRootPasswd)
		RootPasswd = passwd
	}else{
		fmt.Println("非root用户或root密码账户存在")
	}
	// --------------------------- root 配置 -------------------------	


	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"========================开始检测==========================\n" , 0x1B)
	hostname:=host
	var command string
	PortInt:=strconv.Itoa(port)
	for i:=0;i<len(RemoteArg);i++{
		if RemoteArg[i] != "FastScan"{
			command +="-"+RemoteArg[i]+"=all "
		}else{
			command +="-"+RemoteArg[i] + " "
		}
	}
	hostname=strings.TrimSpace(hostname)
	hostname=strings.Replace(hostname, " ", "_", -1)
	todaystr := time.Now().Format("2006-01-02")
	end_time:= time.Now().Format("15:04:05")
	TimeData:="_"+todaystr+"_"+end_time

	commandShell:="cd genmai/src/ ;" + "./"+binMain + " " + command + "-OutPutJson=" + hostname + TimeData + " -Pdf=" + hostname + TimeData + " -host="+host + " -port="+PortInt
	// fmt.Println(commandShell)
	if RootPasswd!=""{
		commandShell=commandShell + " -rootpwd " + RootPasswd
	}

	commandShell=commandShell+" -user " + user + " -passwd " + passwd
	result:=cliConf.RunShell(commandShell)
	fmt.Println(result)
	// //从服务器中下载文件
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"======================文件获取============================\n" , 0x1B)
	if remoteUpdateStatus == 1 {
		remoteUpdateStatus = 0
		return remoteStatus,""
	}
	for i:=0;i<len(RemoteArg);i++{
		remotePathJson:="/home/"+user+"/genmai/data/Report/"+RemoteArg[i]+"_"+hostname+TimeData+".json"
		loaclPathJson:="../data/Report/"+RemoteArg[i]+"_"+hostname+TimeData+".json"
		fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,remotePathJson , 0x1B)
		fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,loaclPathJson , 0x1B)
		v:=cliConf.Download(remotePathJson, loaclPathJson) //文件下载完毕
		if !v {
			return 1002,""
		}
	}
	remotePathPdf:="/home/"+user+"/genmai/data/Report/"+hostname+TimeData+".pdf"
	loaclPathPdf:="../data/Report/"+hostname+TimeData+".pdf"
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,remotePathPdf , 0x1B)
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,loaclPathPdf , 0x1B)
	v2:=cliConf.Download(remotePathPdf, loaclPathPdf) //文件下载完毕
	if !v2 {
		return 1003,""
	}
	fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"=====================Scan Results========================\n" , 0x1B)
	for i:=0;i<len(RemoteArg);i++{
		loaclPath:="../data/Report/"+RemoteArg[i]+"_"+hostname+TimeData+".json"
		jsonFile, err := os.Open(loaclPath)
		if err != nil {
			fmt.Println(err)
		}
		defer jsonFile.Close()
	 
		jsonData, err := ioutil.ReadAll(jsonFile)
		if err!= nil {
			fmt.Println("error reading json file")
		}
		var jsonvul Json
		json.Unmarshal(jsonData,&jsonvul)
		v:=RemoteArg[i]+":"
		fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33, v, 0x1B)
		if RemoteArg[i]=="FastScan"{
			fmt.Println("VbNums		: ",jsonvul.VbNums)
			fmt.Println("VbLevel_hight 	: ",jsonvul.VbLevel_hight)
			fmt.Println("VbLevel_mid 	: ",jsonvul.VbLevel_mid)
			fmt.Println("VbLevel_low		: ",jsonvul.VbLevel_low)
			fmt.Println("VbLevel_ignore		: ",jsonvul.VbLevel_ignore)

		}else{
			fmt.Println("Time : ",jsonvul.Time)
			fmt.Println("Total number of POCs executed 		: ",jsonvul.ExecPocNums)
			fmt.Println("Number of vulnerabilities fixed 	: ",jsonvul.RCRepairedNums)
			fmt.Println("Number of unfixed vulnerabilities 	: ",jsonvul.RCNotFixedNums)
			fmt.Println("Number of POCs not executed		: ",jsonvul.RCNotExecPocNums)
		}
	}
	fmt.Printf("\n%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"========================END==============================\n" , 0x1B)
	if rootChangeVul == 1 {
		command_copy_shadow := "echo "+passwd+" | sudo -S cp /etc/shadow.bak /etc/shadow"
		cliConf.RunShell(command_copy_shadow)
		command_rm_shadowbk := "echo "+passwd+" | sudo -S rm /etc/shadow.bak"
		cliConf.RunShell(command_rm_shadowbk)
	} 
	// cliConf.RunShell("rm -rf genmai")
	return 1,TimeData
}

func RemoteUpdateData(host string,port int,user string, passwd string)bool{
	remoteUpdateStatus = 1
	cliConf := new(ClientConfig)
	errConnet:=cliConf.createClient(host, port, user, passwd)
	if errConnet !=nil{
		remoteStatus = 101
		return false
	}
	cliConf.uploadDirectory("../data/", "/home/"+user+"/genmai/data")
	remoteStatus = 100
	return true
}
func RemoteUpdateSrc(host string,port int,user string, passwd string)bool{
	remoteUpdateStatus = 1
	cliConf := new(ClientConfig)
	errConnet:=cliConf.createClient(host, port, user, passwd)
	if errConnet !=nil{
		remoteStatus = 101
		return false
	}
	cliConf.uploadDirectory(".", "/home/"+user+"/genmai/src")
	remoteStatus = 100
	return true
}
