package FastScan

import (
	"errors"
	"fmt"
	_ "github.com/go-sql-driver/mysql"
	"sync"
	"database/sql"
	// "runtime"
	"sync/atomic"
	"time"
	"strings"
)

// 定义任务
type Task struct {
	Handler func(vul ...interface{}) //可变参数 ，处理可变参数
	Params  []interface{}          // 定义空接口
}

// 任务池的定义
type Pool struct {
	capacity       uint64
	runningWorkers uint64
	status         int64
	chTask         chan *Task
	sync.Mutex
	PanicHandler func(interface{})
}

// 任务池的构造函数
var ErrInvalidPoolCap = errors.New("invalid pool cap")

const (
	RUNNING = 1
	STOPED  = 0
)

func NewPool(capacity uint64) (*Pool, error) {
	if capacity <= 0 {
		return nil, ErrInvalidPoolCap
	}

	return &Pool{
		capacity: capacity,
		status:   RUNNING,
		// 初始化任务队列
		chTask: make(chan *Task, capacity),
	}, nil

}

func (p *Pool) run() {
	p.runningWorkers++

	go func() {
		defer func() {
			p.runningWorkers--
		}()

		for {
			select { //阻塞等待任务，结束信号到来
			case task, ok := <-p.chTask:
				if !ok { //如果 channel 被关闭， 结束worker 运行
					return
				}
				// 执行任务
				task.Handler(task.Params...)
			}
		}
	}()

}

func (p *Pool) incRunning() {
	atomic.AddUint64(&p.runningWorkers, 1)
}

func (p *Pool) decRunning() {
	atomic.AddUint64(&p.runningWorkers, ^uint64(0))
}

func (p *Pool) GetRunningWorkers() uint64 {
	return atomic.LoadUint64(&p.runningWorkers)
}

func (p *Pool) GetCap() uint64 {
	return p.capacity
}

func (p *Pool) setStatus(status int64) bool {
	p.Lock()
	defer p.Unlock()

	if p.status == status {
		return false
	}

	p.status = status
	return true
}

func (p *Pool) WaitTimeForEachTask(t time.Duration) {
	for len(p.chTask) > 0 {
		time.Sleep(t)
	}
    time.Sleep(t)
}

// 任务池关闭
var ErrPoolAlreadyClosed = errors.New("pool already closed")

func (p *Pool) Close() {
	p.setStatus( STOPED)

	for len(p.chTask) > 0 {
		time.Sleep(1e6)
	}

	close(p.chTask)
}


// 生产任务
func (p *Pool) Put(task *Task) error {
	// 枷锁防止启动多个worker
	p.Lock()
	defer p.Unlock()

	// 执行任务
	if p.GetRunningWorkers() < p.GetCap() { //如果任务池满，则不创建worker
		p.run()
	}

	//将任务推入队列，等待消费
	if p.status == RUNNING {
		p.chTask <- task
	}

	return nil
}

func MakeFastScan(dpkgInfo []string,product string,framwork string) (map[string]interface{},map[string]interface{}){
		//创建任务池
		pool, err := NewPool(32)
        defer pool.Close()

		if err != nil {
			panic(err)
		}
    
    // 配置
    type NumOfResult struct {
        // testnums int
        // testnums1 int
        VbNums          int
        VbLevel_hight int
        VbLevel_mid   int
        VbLevel_low   int
        VbLevel_unknown int
        VbLevel_fatal int
        // VbLevel_ignore = 0
        VbMutex sync.Mutex

        NumOfFinished int
    }

    var numOfRes NumOfResult
    numOfRes.VbNums=0
    numOfRes.VbLevel_hight=0
    numOfRes.VbLevel_mid=0
    numOfRes.VbLevel_low=0
    numOfRes.VbLevel_unknown=0
    numOfRes.VbLevel_fatal=0
    numOfRes.NumOfFinished =0

    dbusername, dbpassword, dbhostsip, dbname:=ConfigYamlParse()
    dbinfo := strings.Join([]string{dbusername, ":", dbpassword, "@tcp(", dbhostsip, ")/", dbname, "?charset=utf8"}, "")
    db,err:=sql.Open("mysql",dbinfo)
    // db,err:=sql.Open("sqlite",dbinfo)
    checkerr(err)
    // 设置最大连接数
    db.SetConnMaxLifetime(2000)
    // 设置数据库最大的闲置连接数
    db.SetMaxIdleConns(2000)
    db.SetMaxOpenConns(2000)

    //>>>dbInfo
    type info struct {
        id int `db:"id"`
        cve_no string `db: "cve_no"`
        role_level string `db: "role_level"`
        ubuntu_v4_status string `db: "ubuntu_v4_status"`
        version_v4_status string `db: "version_v4_status"`
        ubuntu_v4_edition string `db: "ubuntu_v4_edition"`
        kylin_v4_edition sql.NullString `db: "kylin_v4_edition"`
        ubuntu_v10_status string `db: "ubuntu_v10_status"`
        version_v10_status string `db: "version_v10_status"`
        ubuntu_v10_edition string `db: "ubuntu_v10_edition"`
        kylin_v10_edition sql.NullString `db: "kylin_v10_edition"`
        ubuntu_v10_1_status string `db: "ubuntu_v10_1_status"`
        version_v10_1_status string `db: "version_v10_1_status"`
        ubuntu_v10_1_edition string `db: "ubuntu_v10_1_edition"`
        kylin_v10_1_edition sql.NullString `db: "kylin_v10_1_edition"`
        version_ubuntu_kylin_status sql.NullString `db: "version_ubuntu_kylin_status"`
        ubuntu_kylin_edition sql.NullString `db: "ubuntu_kylin_edition"`
        package_name string `db: "package_name"`
        publish_status string `db: "publish_status"`
        date string `db: "date"`
    }

    //预编译处理查询数据防止sql注入
    sqlStr :="SELECT * FROM kvsp_cvelist where package_name = ?"
    stmt, err := db.Prepare(sqlStr)
    checkerr(err)
    // defer stmt.Close()
    // var packagesList [][]string
    // var packagesOn map[string]interface{}
    // packagesOn = make(map[string]interface{})
    // var packagesList map[string]map[string]interface{}
    packagesList := make(map[string]map[string]interface{})
    packagesList_notin := make(map[string]map[string]interface{})

    // var wg sync.WaitGroup


    // threads_num := 200

    // 系统版本匹配
    var opInfo = product
    var barLength int

    var NumOfPutted = 0
    for i:=0;i<len(dpkgInfo);i++{
        if FastScanFalseAlarmConfigYamlParse(dpkgInfo[i]){
            continue
        }
        // if i%2 != 0 {
        //     continue
        // }
        v := "def"
        var cve_id []string
        var cve_name []string
        var bin_name []string
        var cve_risk []string
        var cve_status []string
        var cve_type []string
        var cve_introduction []string
        var cve_currentVersion []string
        var cve_secureVersion []string
        var cve_repairSuggestions []string
        var cve_n_id []string
        var cve_n_name []string
        var cve_n_risk []string
        var cve_n_status []string
        var cve_n_currentVersion []string
        var cve_n_secureVersion []string

        barLength = len(dpkgInfo)-1


        time.Sleep(time.Millisecond)
        // wg.Add(1)
        // go func(i int ) {
        NumOfPutted++
		pool.Put(&Task{
				Handler: func(vul ...interface{}) {
            // defer wg.Done()
            _i := vul[0].(int)
            if _i & 1 == 0 {
                rows, err := stmt.Query(dpkgInfo[_i])
                checkerr(err)
                defer rows.Close()
                // numOfRes.VbMutex.Lock()
                // numOfRes.testnums++
                // numOfRes.VbMutex.Unlock()

                for rows.Next(){
                    // numOfRes.VbMutex.Lock()
                    // numOfRes.testnums1++
                    // numOfRes.VbMutex.Unlock()
                    var s info
					var dpkgInfoStr string
                    err = rows.Scan(&s.id,&s.cve_no,&s.role_level,&s.ubuntu_v4_status,&s.version_v4_status,&s.ubuntu_v4_edition,&s.kylin_v4_edition,&s.ubuntu_v10_status,&s.version_v10_status,&s.ubuntu_v10_edition,&s.kylin_v10_edition,&s.ubuntu_v10_1_status,&s.version_v10_1_status,&s.ubuntu_v10_1_edition,&s.kylin_v10_1_edition,&s.version_ubuntu_kylin_status,&s.ubuntu_kylin_edition,&s.package_name,&s.publish_status ,&s.date)
                    checkerr(err)

					dpkgInfoStr=dpkgInfo[_i+1]
					
                    
                    var result int = 100

                                            // ----------------查询cve的详情------------
                                            var roleName string
                                            var role_introduction string
                                            var role_type string
                                            sqlStr :="SELECT role_name,role_introduction,role_type FROM kvsp_cveinfo WHERE cve_no = ?"
                                            _stmt, err := db.Prepare(sqlStr)
                                            if err != nil {
                                                fmt.Println("_stmt:",err)
                                                // _stmt.Close()
                                                continue    
                                            }
                                            
                                            _rows, err := _stmt.Query(s.cve_no)
                                            if err != nil {
                                                fmt.Println("_row:",err)
                                                // _rows.Close()
                                                continue     
                                            }
                                            // time.Sleep(100 *time.Millisecond)
                                            for _rows.Next(){
                                                err = _rows.Scan(&roleName,&role_introduction,&role_type)
                                                if err != nil {
                                                    fmt.Println("_row_next:",err)
                                                    // _stmt.Close()
                                                    // _rows.Close()
                                                    continue     
                                                }
                                            }
                                            _stmt.Close()
                                            _rows.Close()
                                            // ----------------查询cve的详情------------
                    if dpkgInfoStr != "" && s.ubuntu_v10_1_edition != "" && opInfo == "PROJECT_CODENAME=V10SP1" && s.ubuntu_v10_1_status == "Released"{
                        result=PushCompareVersion(s.cve_no,dpkgInfoStr,s.ubuntu_v10_1_edition,0)
                    }
                    if dpkgInfoStr != "" && s.ubuntu_v10_edition != "" && opInfo == "PROJECT_CODENAME=V10" && s.ubuntu_v10_status == "Released"{
                        result=PushCompareVersion(s.cve_no,dpkgInfoStr,s.ubuntu_v10_edition,1)
                    }
                    if dpkgInfoStr != "" && s.ubuntu_v4_edition != "" && opInfo == "PROJECT_CODENAME=V4" && s.ubuntu_v4_status == "Released"{
                        result=PushCompareVersion(s.cve_no,dpkgInfoStr,s.ubuntu_v4_edition,1)
                    }
                    if result==2{
                        SourceCode:=getSourceCode(dpkgInfo[_i])
                        if SourceCode == "" {
                            SourceCode = dpkgInfo[_i]
                        }
                        var secureVersion string
                        v = SourceCode
                        cve_id =append(cve_id,s.cve_no)
                        cve_currentVersion = append(cve_currentVersion,dpkgInfoStr)
                        cve_name = append(cve_name,roleName)
                        cve_introduction = append(cve_introduction,role_introduction)
                        cve_type = append(cve_type,role_type)
                        bin_name = append(bin_name,dpkgInfo[_i])
                        cve_risk = append(cve_risk,s.package_name)
                        cve_status = append(cve_status,s.version_v10_1_status)
                        if opInfo == "PROJECT_CODENAME=V10SP1" {
                            secureVersion = strings.Replace(s.ubuntu_v10_1_edition,"ubuntu","kylin",-1)
                            cve_secureVersion = append(cve_secureVersion,secureVersion)
                        }else{
                            secureVersion = strings.Replace(s.ubuntu_v10_1_edition,"ubuntu","kord",-1)
                            cve_secureVersion = append(cve_secureVersion,secureVersion)
                        }
                        if s.version_v10_1_status == "已修复"{
                            vul:="# 使用apt/yum install " + v + " 更新到"+secureVersion+"安全版本"
                            cve_repairSuggestions = append(cve_repairSuggestions,vul)
                        }else{
                            vul:="# 请联系技服团队解决该问题"
                            cve_repairSuggestions = append(cve_repairSuggestions,vul)
                        }
						numOfRes.VbMutex.Lock()
                        numOfRes.VbNums = numOfRes.VbNums+1


                        if s.package_name == "高危"{
                            numOfRes.VbLevel_hight = numOfRes.VbLevel_hight + 1
                        }
                        if s.package_name == "中危"{
                            numOfRes.VbLevel_mid = numOfRes.VbLevel_mid + 1
                        }
                        if s.package_name == "低危"{
                            numOfRes.VbLevel_low = numOfRes.VbLevel_low + 1
                        }
                        if s.package_name == "未知"{
                            numOfRes.VbLevel_unknown = numOfRes.VbLevel_unknown + 1
                        }
                        if s.package_name == "致命"{
                            numOfRes.VbLevel_fatal = numOfRes.VbLevel_fatal + 1
                        }
                        numOfRes.VbMutex.Unlock()
                    }else {
                        if s.ubuntu_v10_1_status == "Released" || s.ubuntu_v10_1_status == "Does not exist" || s.ubuntu_v10_1_status =="Ignore"{
                            cve_n_id =append(cve_n_id,s.cve_no)
                            cve_n_currentVersion = append(cve_n_currentVersion,dpkgInfoStr)
                            cve_n_name = append(cve_n_name,roleName)
                            cve_n_risk = append(cve_n_risk,s.package_name)
                            cve_n_status = append(cve_n_status,s.version_v10_1_status)
                            if opInfo == "PROJECT_CODENAME=V10SP1" {
                                secureVersion := strings.Replace(s.ubuntu_v10_1_edition,"ubuntu","kylin",-1)
                                cve_n_secureVersion = append(cve_n_secureVersion,secureVersion)
                            }else{
                                secureVersion := strings.Replace(s.ubuntu_v10_1_edition,"ubuntu","kord",-1)
                                cve_n_secureVersion = append(cve_n_secureVersion,secureVersion)
                            }
                        }
                    } // if result ...

                } // for rows ...

                // rows.Close()
                if v != "def"{
                    var header string
                    SourceCode:=getSourceCode(dpkgInfo[_i])
                    if SourceCode == "" {
                        SourceCode = dpkgInfo[_i]
                    }
                    // 查询包的负责人
                    sqlStr :="SELECT header FROM package_info WHERE package_name = ?"
                    __stmt, err := db.Prepare(sqlStr)
                    if err != nil {
                        fmt.Println(4,err) 
                    }
                    __rows, err := __stmt.Query(SourceCode)
                    for __rows.Next(){
                        err = __rows.Scan(&header)
                        if err != nil {
                            fmt.Println(5,err)    
                            continue
                        }
                    }
                    __stmt.Close()
                    __rows.Close()
                    // 存在的漏洞信息存储
                    // numOfRes.VbMutex.Lock()
                    packagesList[v] = make(map[string]interface{})
                    packagesList[v]["bin_name"] = bin_name
                    packagesList[v]["cve_id"] = cve_id
                    packagesList[v]["cve_name"] = cve_name
                    packagesList[v]["cve_introduction"] = cve_introduction
                    packagesList[v]["cve_type"] = cve_type
                    packagesList[v]["cve_status"] = cve_status
                    packagesList[v]["cve_risk"] = cve_risk
                    packagesList[v]["cve_currentVersion"] = cve_currentVersion
                    packagesList[v]["cve_secureVersion"] = cve_secureVersion
                    packagesList[v]["cve_repairSuggestions"] = cve_repairSuggestions
                    packagesList[v]["header"] = header
                    packagesList[v]["send_email_commit"] = "xxxxxx"

                    // 不存在的漏洞信息存储
                    packagesList_notin[v] = make(map[string]interface{})
                    packagesList_notin[v]["cve_n_id"] = cve_n_id
                    packagesList_notin[v]["cve_n_name"] = cve_n_name
                    packagesList_notin[v]["cve_n_risk"] = cve_n_risk
                    packagesList_notin[v]["cve_n_status"] = cve_n_status
                    packagesList_notin[v]["cve_n_currentVersion"] = cve_n_currentVersion
                    packagesList_notin[v]["cve_n_secureVersion"] = cve_n_secureVersion
                    // numOfRes.VbMutex.Unlock()                    
                }
            } // if & 1 ..
            // wg.Wait()
        // } (_i) // go func
        numOfRes.VbMutex.Lock()
        numOfRes.NumOfFinished++
        numOfRes.VbMutex.Unlock()
	},
	Params: []interface{}{i},
})
        // wg.Wait()
    ProgressBar(barLength, numOfRes.NumOfFinished)
    // fmt.Print(">><<A", pool.GetRunningWorkers() )
    } //for

    for i:=0;i<600 ;i++{
        if numOfRes.NumOfFinished < NumOfPutted {
            time.Sleep(time.Second)
        } else {
            break
        }
        ProgressBar(barLength, numOfRes.NumOfFinished)
    }
    fmt.Println("\n")
    fmt.Printf("%c[%d;%d;%dm%s%c[0m\n", 0x1B, 0, 0, 33,"=====================Scan Results======================\n" , 0x1B)
    fmt.Println("共计漏洞：",numOfRes.VbNums)
    fmt.Println("致命漏洞：",numOfRes.VbLevel_fatal)
    fmt.Println("高危漏洞：",numOfRes.VbLevel_hight)
    fmt.Println("中危漏洞：",numOfRes.VbLevel_mid)
    fmt.Println("低危漏洞：",numOfRes.VbLevel_low)
    fmt.Println("未知漏洞：",numOfRes.VbLevel_unknown)
    // fmt.Println("<<<：",cve_id)

    //////////////////////////////
    //获取系统信息
    // _,arch := GetCurrentFw()
    // product := GetdOpInfo()
    // framwork :=GetdOpUInfo()
    //////////////////////////////
    var resultMap map[string]interface{}
    var package_info map[string]interface{}
    var packages map[string]interface{}
    var packages_notin map[string]interface{}

    var iso_url = []string{"iso1_url", "iso2_url"}
    var relate_url = []string{"https://www.kylinos.cn/support/loophole/patch.html","http://archive.kylinos.cn/kylin/KYLIN-ALL/"}
    
    resultMap = make(map[string]interface{})
    package_info = make(map[string]interface{})
    packages = make(map[string]interface{})
    packages_notin = make(map[string]interface{})
    // package_info -> map 
    package_info["product"] = product
    package_info["iso_url"] = iso_url
    package_info["relate_url"] = relate_url

    // packages -> map
    packages["arch"] = framwork
    packages["packagesList"] = packagesList
    packages_notin["packagesList"] = packagesList_notin
    
    // packages -> package_info 
        // package_info -> map 
        package_info["product"] = product
        package_info["iso_url"] = iso_url
        package_info["relate_url"] = relate_url
    package_info["packages"]=packages

    // package_info -> resultMap
    resultMap["package_info"]=package_info
    
    // resultMap -> map
    resultMap["source_url"]="xxxxxx"
    resultMap["VbNums"]=numOfRes.VbNums
    resultMap["VbLevel_hight"]=numOfRes.VbLevel_hight
    resultMap["VbLevel_mid"]=numOfRes.VbLevel_mid
    resultMap["VbLevel_low"]=numOfRes.VbLevel_low
    // resultMap["numOfRes.VbLevel_ignore"]=numOfRes.VbLevel_ignore
    resultMap["VbLevel_fatal"]=numOfRes.VbLevel_fatal
    resultMap["VbLevel_unknown"]=numOfRes.VbLevel_unknown


    
    return resultMap,packages_notin
}
